﻿using UnityEngine;
using System.Collections.Generic;

public class Building : MonoBehaviour
{
	public BuildingPrefabSet prefabSet;
	public BuildingPrefabSet[] specialPrefabSets;


//	public GameObject[] cornerPrefabs;
//	public Material[] floorMaterials;
//	public Material[] ceilingMaterials;
//	public Material[] roofMaterials;
//	public string[] outerWallPatterns;
//	public GameObject[] outerWallPrefabs;
//	public GameObject[] innerWallPrefabs;
//
//	public GameObject[] specialPrefabs;
//	public GameObject[] specialCornerPrefabs;
//	public Material[] specialFloorMaterials;
//	public Material[] specialCeilingMaterials;
//	public string[] specialOuterWallPatterns;

	public int specialId;
	public int specialFloor;
	public bool varyFloorSize;

	public Vector3 sizeInUnits;

	public List<Vector3> possibleItemLocations;

	public Vector3 GetMaxSize()
	{
		return new Vector3((float)(sizeInUnits.x * 2), (float)(sizeInUnits.y * 3), (float)(sizeInUnits.z * 2));
	}

	void Start () 
	{
		possibleItemLocations = new List<Vector3>();
		specialFloor = Random.Range(0, (int)sizeInUnits.y);

		int x1 = Random.Range(1, varyFloorSize ? (int)sizeInUnits.x / 4: 1);
		int x2 = (int)sizeInUnits.x - 1 - Random.Range(1, varyFloorSize ? (int)sizeInUnits.x / 4: 1);
		int z1 = Random.Range(1, varyFloorSize ? (int)sizeInUnits.z / 4: 1);
		int z2 = (int)sizeInUnits.z - 1 - Random.Range(1, varyFloorSize ? (int)sizeInUnits.z / 4: 1);

		//Debug.Log(x1.ToString() + ":" + x2.ToString() + ":" + z1.ToString() + ":" + z2.ToString());

		for(int i = 0; i < (int)sizeInUnits.y; i++)
		{
			GenerateStory(i, x1, x2, z1, z2);

			if(varyFloorSize)
			{
				int sizeX = x2 - x1 + 1;
				int sizeZ = z2 - z1 + 1;

				if(sizeX > 3 && Random.value < 0.06f)
				{
					var inset = Random.Range(1, sizeX - 3);
					GenerateRoof(i, x1, x1 + inset - 1, z1, z2);
					x1 += inset;
				}
				if(sizeX > 3 && Random.value < 0.06f)
				{
					var inset = Random.Range(1, sizeX - 3);
					GenerateRoof(i, x2 - inset + 1, x2, z1, z2);
					x2 -= inset;
				}
				if(sizeZ > 3 && Random.value < 0.06f)
				{
					var inset = Random.Range(1, sizeZ - 3);
					GenerateRoof(i, x1, x2, z1, z1 + inset - 1);
					z1 += inset;
				}
				if(sizeZ > 3 && Random.value < 0.06f)
				{
					var inset = Random.Range(1, sizeZ - 3);
					GenerateRoof(i, x1, x2, z2 - inset + 1, z2);
					z2 -= inset;
				}
			}
		}

		GenerateRoof((int)sizeInUnits.y - 1, x1, x2, z1, z2);
	}
	
	private void GenerateStory(int floorNumber, int x1, int x2, int z1, int z2)
	{
		var prefabSet = GetPrefabSet(floorNumber);

		GenerateFloorAndCeiling(prefabSet, floorNumber, x1, x2, z1, z2);
		GenerateCorners(prefabSet, floorNumber, x1, x2, z1, z2);
		GenerateOuterWallX(prefabSet, floorNumber, x1, z1, z2, 90f, Vector3.forward * 2f);
		GenerateOuterWallX(prefabSet, floorNumber, x2, z1, z2, 270f, Vector3.right * 2f);
		GenerateOuterWallZ(prefabSet, floorNumber, z1, x1, x2, 0f, Vector3.zero);
		GenerateOuterWallZ(prefabSet, floorNumber, z2, x1, x2, 180f, new Vector3(2f, 0f, 2f));

		if(IsSpecialFloor(floorNumber))
		{
			GenerateSpecial(prefabSet, floorNumber, x1, x2, z1, z2);
		}
		else
		{
			var basePosition = (GetPosition(floorNumber, x1, z1) + GetPosition(floorNumber, x2 + 1, z2 + 1)) * 0.5f;
			possibleItemLocations.Add(basePosition);
		}
//
//		// Generate inner walls
		//		SubDivide(prefabSet, floorNumber, x1, x2, z1, z2);
	}

	private void SubDivide(BuildingPrefabSet prefabSet, int floorNumber, int x1, int x2, int z1, int z2)
	{
		int sizeX = (x2 - x1);
		int sizeZ = (z2 - z1);
		int area = sizeX * sizeZ;
		if(Random.Range(0, 100) < area)
		{
			int sum = sizeX + sizeZ;
			if(Random.Range(0, sum) < sizeX)
			{
				if(sizeX > 4)
				{
					int x = Random.Range(0, sizeX / 4) + (sizeX / 2);
					GenerateInnerWallX(prefabSet, floorNumber, x, z1, z2);
					SubDivide(prefabSet, floorNumber, x1, x, z1, z2);
					SubDivide(prefabSet, floorNumber, x, x2, z1, z2);
				}
			}
			else
			{
				if(sizeZ > 4)
				{
					int z = Random.Range(0, sizeZ / 4) + (sizeZ / 2);
					GenerateInnerWallZ(prefabSet, floorNumber, z, x1, x2);
					SubDivide(prefabSet, floorNumber, x1, x2, z1, z);
					SubDivide(prefabSet, floorNumber, x1, x2, z, z2);
				}
			}
		}
	}

	private void GenerateFloorAndCeiling(BuildingPrefabSet prefabSet, int floorNumber, int x1, int x2, int z1, int z2)
	{
		var floorMaterial = ArrayUtility.GetRandomElement(prefabSet.floorMaterials);
		var ceilingMaterial = ArrayUtility.GetRandomElement(prefabSet.ceilingMaterials);

		GenerateFloor(floorNumber, x1, x2, z1, z2, floorMaterial);
		GenerateCeiling(floorNumber, x1, x2, z1, z2, ceilingMaterial);
	}

	private void GenerateRoof(int floorNumber, int x1, int x2, int z1, int z2)
	{
		var floorMaterial = ArrayUtility.GetRandomElement(prefabSet.roofMaterials);
		GenerateRoof(floorNumber, x1, x2, z1, z2, floorMaterial);
	}

	private void GenerateCorners(BuildingPrefabSet prefabSet, int floorNumber, int x1, int x2, int z1, int z2)
	{
		var prefab = ArrayUtility.GetRandomElement(prefabSet.cornerPrefabs);

		GenerateCorner(floorNumber, x1, z1, prefab);
		GenerateCorner(floorNumber, x2, z1, prefab);
		GenerateCorner(floorNumber, x1, z2, prefab);
		GenerateCorner(floorNumber, x2, z2, prefab);
	}

	private void GenerateSpecial(BuildingPrefabSet prefabSet, int floorNumber, int x1, int x2, int z1, int z2)
	{
		//var basePosition = new Vector3(((float)x1 + (float)(x2 + 1)) * 0.5f, (float)(floorNumber * 4), ((float)z1 + (float)(z2 + 1)) * 0.5f);
		var basePosition = (GetPosition(floorNumber, x1, z1) + GetPosition(floorNumber, x2 + 1, z2 + 1)) * 0.5f;

//		if(prefabSet == null)
//		{
//			Debug.Log("Boo1");
//		}
//
//		if(prefabSet.specialPrefabs == null)
//		{
//			Debug.Log("Boo2");
//		}

		if(prefabSet.specialPrefabs != null && prefabSet.specialPrefabs.Length > 0)
		{
			var prefab = ArrayUtility.GetRandomElement(prefabSet.specialPrefabs);

			if(prefab != null)
			{
				var meshObject = Instantiate(prefab, basePosition, Quaternion.identity) as GameObject;
				meshObject.transform.parent = gameObject.transform;
				meshObject.transform.localPosition = basePosition;
			}
		}
	}

	private void GenerateFloor(int floorNumber, int x1, int x2, int z1, int z2, Material material)
	{
		var basePosition = GetPosition(floorNumber, x1, z1);
		var uvX = (float)((x2 - x1 + 1) * 2);
		var uvZ = (float)((z2 - z1 + 1) * 2);
		var mesh = new Mesh();
		mesh.vertices = new Vector3[] { new Vector3(basePosition.x, basePosition.y + 0.1f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 0.1f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 0.1f, basePosition.z + uvZ),
			new Vector3(basePosition.x, basePosition.y + 0.1f, basePosition.z + uvZ)
		};
		mesh.normals = new Vector3[] { Vector3.up, Vector3.up, Vector3.up, Vector3.up };
		mesh.uv = new Vector2[] { Vector2.zero,
			new Vector2(uvX, 0f),
			new Vector2(uvX, uvZ),
			new Vector2(0f, uvZ)
		};
		mesh.triangles = new int[] { 0, 3, 2, 2, 1, 0} ;
		mesh.RecalculateBounds();

		CreateMeshGameObject(mesh, transform, material, "Story " + floorNumber + " Floor");
	}
	
	private void GenerateRoof(int floorNumber, int x1, int x2, int z1, int z2, Material material)
	{
		var basePosition = GetPosition(floorNumber, x1, z1);
		var uvX = (float)((x2 - x1 + 1) * 2);
		var uvZ = (float)((z2 - z1 + 1) * 2);
		var mesh = new Mesh();
		mesh.vertices = new Vector3[] { new Vector3(basePosition.x, basePosition.y + 4f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 4f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 4f, basePosition.z + uvZ),
			new Vector3(basePosition.x, basePosition.y + 4f, basePosition.z + uvZ)
		};
		mesh.normals = new Vector3[] { Vector3.up, Vector3.up, Vector3.up, Vector3.up };
		mesh.uv = new Vector2[] { Vector2.zero,
			new Vector2(uvX, 0f),
			new Vector2(uvX, uvZ),
			new Vector2(0f, uvZ)
		};
		mesh.triangles = new int[] { 0, 3, 2, 2, 1, 0} ;
		mesh.RecalculateBounds();
		
		CreateMeshGameObject(mesh, transform, material, "Story " + floorNumber + " Roof");
	}
	
	private void GenerateCeiling(int floorNumber, int x1, int x2, int z1, int z2, Material material)
	{
		var basePosition = GetPosition(floorNumber, x1, z1);
		var uvX = (float)((x2 - x1 + 1) * 2);
		var uvZ = (float)((z2 - z1 + 1) * 2);
		//Debug.Log(basePosition.ToString() + ":" + uvX + ":" + uvZ);
		var mesh = new Mesh();
		mesh.vertices = new Vector3[] { new Vector3(basePosition.x, basePosition.y + 3.7f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 3.7f, basePosition.z),
			new Vector3(basePosition.x + uvX, basePosition.y + 3.7f, basePosition.z + uvZ),
			new Vector3(basePosition.x, basePosition.y + 3.7f, basePosition.z + uvZ)
		};
		mesh.normals = new Vector3[] { Vector3.down, Vector3.down, Vector3.down, Vector3.down };
		mesh.uv = new Vector2[] { Vector2.zero,
			new Vector2(uvX * 0.5f, 0f),
			new Vector2(uvX * 0.5f, uvZ * 0.5f),
			new Vector2(0f, uvZ * 0.5f)
		};
		mesh.triangles = new int[] { 0, 1, 2, 2, 3, 0} ;
		mesh.RecalculateBounds();
		
		CreateMeshGameObject(mesh, transform, material, "Story " + floorNumber + " Ceiling");
	}

	private void GenerateCorner(int floorNumber, int x, int z, GameObject prefab)
	{
		var basePosition = GetPosition(floorNumber, x, z);

		var meshObject = Instantiate(prefab, basePosition, Quaternion.identity) as GameObject;
		meshObject.transform.parent = gameObject.transform;
		meshObject.transform.localPosition = basePosition;
	}

	private void GenerateWall(int floorNumber, int x, int z, float angle, Vector3 offset, GameObject prefab)
	{
		var basePosition = GetPosition(floorNumber, x, z);
		
		var meshObject = Instantiate(prefab, basePosition, Quaternion.Euler(0f, angle, 0f)) as GameObject;
		meshObject.transform.parent = gameObject.transform;
		meshObject.transform.position = gameObject.transform.position + basePosition + offset;
	}

	private void GenerateOuterWallX(BuildingPrefabSet prefabSet, int floorNumber, int x, int z1, int z2, float angle, Vector3 offset)
	{
		int baseLeft = z1 + 1;
		int baseRight = z2 - 1;
		int index = 0;

		var pattern = ArrayUtility.GetRandomElement(prefabSet.outerWallPatterns);
		while(baseLeft <= baseRight)
		{
			char elementChar = pattern[index];
			int element = elementChar - 65;

			var prefab = prefabSet.outerWallPrefabs[element];

			GenerateWall(floorNumber, x, baseLeft, angle, offset, prefab);
			if(baseRight != baseLeft)
			{
				GenerateWall(floorNumber, x, baseRight, angle, offset, prefab);
			}

			index++;
			if(Random.value < 0.01f)
			{
				index++;
			}
			if(index >= pattern.Length)
			{
				index = 0;
			}
			baseLeft++;
			baseRight--;

		}
	}

	private void GenerateOuterWallZ(BuildingPrefabSet prefabSet, int floorNumber, int z, int x1, int x2, float angle, Vector3 offset)
	{
		int baseLeft = x1 + 1;
		int baseRight = x2 - 1;
		int index = 0;
		
		var pattern = ArrayUtility.GetRandomElement(prefabSet.outerWallPatterns);
		while(baseLeft <= baseRight)
		{
			char elementChar = pattern[index];
			int element = elementChar - 65;
			
			var prefab = prefabSet.outerWallPrefabs[element];
			
			GenerateWall(floorNumber, baseLeft, z, angle, offset, prefab);
			if(baseRight != baseLeft)
			{
				GenerateWall(floorNumber, baseRight, z, angle, offset, prefab);
			}
			
			index++;
			if(Random.value < 0.01f)
			{
				index++;
			}
			if(index >= pattern.Length)
			{
				index = 0;
			}
			baseLeft++;
			baseRight--;
			
		}
	}

	private void GenerateInnerWallX(BuildingPrefabSet prefabSet, int floorNumber, int x, int z1, int z2)
	{
		//TODO: 
	}
	
	private void GenerateInnerWallZ(BuildingPrefabSet prefabSet, int floorNumber, int z, int x1, int x2)
	{
		//TODO: 
	}

	private bool IsSpecialFloor(int floor)
	{
		if(specialFloor == floor)
		{
			return true;
		}
		return false;
	}

	private Vector3 GetPosition(int floorNumber, int x, int z)
	{
		return new Vector3((float)(x * 2) - ((float)sizeInUnits.x), (float)(floorNumber * 4), (float)(z * 2) - ((float)sizeInUnits.z));
	}

	private BuildingPrefabSet GetPrefabSet(int floorNumber)
	{
		if(!IsSpecialFloor(floorNumber))
		{
			return prefabSet;
		}
		//Debug.Log (specialId);
		return specialPrefabSets[specialId];
	}

	private GameObject CreateMeshGameObject(Mesh mesh, Transform transform, Material material, string name)
	{
		var meshObject = new GameObject(name);
		meshObject.hideFlags = HideFlags.DontSave;
		var meshFilter = meshObject.AddComponent<MeshFilter>();
		var meshRenderer = meshObject.AddComponent<MeshRenderer>();
		var meshCollider = meshObject.AddComponent<MeshCollider>();
		meshObject.transform.position = gameObject.transform.position;
		meshObject.transform.parent = gameObject.transform;
		meshObject.layer = gameObject.layer;

		meshRenderer.renderer.material = material;
		meshFilter.mesh = mesh;
		meshCollider.sharedMesh = mesh;

		return meshObject;
	}
}
