﻿using UnityEngine;
using System.Collections.Generic;
using System.Linq;

public class BuildingFactory : MonoBehaviour 
{
	public int numBuildings = 20;
	public float levelRadius = 90f;

	public GameObject zeusBuildingPrefab;
	public GameObject zeusPortalPrefab;
	public GameObject[] buildingPrefabs;
	public GameObject[] keyPrefabs;

	public BuildingPrefabSet[] specialPrefabSets;
	public int[] keyCounts;

	public List<Building> buildings;
	public bool itemsSeeded;

	// Use this for initialization
	void Start () 
	{
		GenerateBuildings();
	}
	
	// Update is called once per frame
	void Update () 
	{
		if(!itemsSeeded)
		{
			SeedItems();
		}
	}

	public void GenerateBuildings()
	{
		var buildingBounds = new List<Bounds>();
		buildings = new List<Building>();

		int specialId = 4;

		for(int i = 0; i < numBuildings + 1; i++)
		{
			var chosenPrefab = i < numBuildings ? ArrayUtility.GetRandomElement(buildingPrefabs) : zeusBuildingPrefab;
			var building = chosenPrefab.GetComponent<Building>();
			if(building == null)
			{
				Debug.LogWarning("Prefab '" + chosenPrefab.name + "' was in the list of building prefabs, but was not a building");
			}

			var size = building.GetMaxSize();
			var center = new Vector3(0f, size.y * 0.5f, 0f);

			Vector3 position = Vector3.zero;
			if(i > 0)
			{

				bool isValidPosition = false;
				while(!isValidPosition)
				{
					var inCircle = Random.insideUnitCircle * levelRadius;
					position = i < numBuildings ? new Vector3(inCircle.x, 0f, inCircle.y) : new Vector3(0f, -40f, 0f);

					var newBound = new Bounds(position + center, size);
					isValidPosition = true;
					foreach(var bound in buildingBounds)
					{
						if(bound.Intersects(newBound))
						{
							isValidPosition = false;
							break;
						}
					}

					if(isValidPosition)
					{
						buildingBounds.Add(newBound);
					}
				}
			}
			else
			{
				var newBound = new Bounds(position + center, size);
				buildingBounds.Add(newBound);
			}

			var go = Instantiate(chosenPrefab, position, Quaternion.identity) as GameObject;
			var buildingInstance = go.GetComponent<Building>();
			buildingInstance.specialPrefabSets = specialPrefabSets;
			if(i > 0)
			{
				buildingInstance.varyFloorSize = i < numBuildings;
				buildingInstance.specialId = i < numBuildings ? specialId : 5;
				if(specialId > 0)
				{
					specialId--;
				}
			}
			else
			{
				buildingInstance.sizeInUnits.y = 7f;
			}
			buildings.Add(buildingInstance);
		}

	}

	public void SeedItems()
	{
		var allPossibleItemLocations = buildings.SelectMany(i => i.possibleItemLocations, (j, k) => j.transform.position + k).ToList();
		//Debug.Log(allPossibleItemLocations.Count);
		if(allPossibleItemLocations.Count == 0)
		{
			return;
		}

		for(int i = 0; i < keyCounts.Length; i++)
		{
			for(int j = 0; j < keyCounts[i]; j++)
			{
				var location = ArrayUtility.GetRandomElementAndRemove(allPossibleItemLocations);
				
				var position = location + Vector3.up;
				Instantiate(keyPrefabs[i], position, Quaternion.identity);
			}
		}

		itemsSeeded = true;
	}

	public void SpawnZeusKey()
	{
		Instantiate(keyPrefabs[5], new Vector3(0f, -39f, 0f), Quaternion.identity);
		Instantiate(zeusPortalPrefab, new Vector3(0f, 0f, 0f), Quaternion.identity);
		Messenger.SetMessage("The way to the final key is now open. Retrieve it to claim your destiny as Zeus's Champion!");
	}

}
