﻿using UnityEngine;
using System.Collections;
using System.Linq;

public class EpicEnd : MonoBehaviour
{
	public GameObject waterPrefab;
	public GameObject splashPrefab;

	public Player player;
	public GameObject submarine;
	public CameraController cameraController;
	public BuildingFactory buildingFactory;
	public GameObject water;
	public Light dirLight;
	public float timer;
	public bool endMessageShown;
	public bool waterPrefabSpawned;
	public bool splashSpawned;

	void Start () 
	{
		player = FindObjectOfType<Player>();
		player.AddAir(720f);
		player.rigidbody.useGravity = false;
		player.gameIsWon = true;
		cameraController = FindObjectOfType<CameraController>();
		dirLight = FindObjectsOfType<Light>().Where(i => i.type == LightType.Directional).FirstOrDefault();
		timer = 0f;

		var shark = FindObjectOfType<Shark>();
		Destroy(shark.gameObject);

		var cityElements = GameObject.FindGameObjectsWithTag("CityElement");
		//Debug.Log(cityElements.Length);
		foreach(var cityElement in cityElements)
		{
			cityElement.transform.parent = transform;
		}
//		var factory = GameObject.FindObjectOfType<BuildingFactory>();
//		Debug.Log(factory.buildings.Count);
//		foreach(var building in factory.buildings)
//		{
//			building.transform.parent = transform;
//		}

		submarine = GameObject.FindGameObjectWithTag("Submarine");
		submarine.transform.parent = transform;

	}
	
	void Update () 
	{
		timer += Time.deltaTime;

		if(!waterPrefabSpawned)
		{
			water = Instantiate(waterPrefab, new Vector3(0f, 190f, 0f), Quaternion.identity) as GameObject;
			waterPrefabSpawned = true;
		}

		if(timer < 15f)
		{
			//Let player read message
		}
		else if(timer < 55f)
		{
			if(!splashSpawned)
			{
				Instantiate(splashPrefab, new Vector3(0f, 190f, 0f), Quaternion.Euler(270f, 0f, 0f));
				splashSpawned = true;
			}

			cameraController.enabled = false;

			Camera.main.transform.position = new Vector3(-40f, 240f,-40f);
			Camera.main.transform.localRotation = Quaternion.Euler(30f, 45f, 0f);
			Camera.main.farClipPlane = 200f;
			Camera.main.backgroundColor = new Color(0.2f, 1.0f, 1.0f);
			RenderSettings.ambientLight = new Color(0.4f, 0.4f, 0.25f);
			RenderSettings.fogDensity = 0.01f;
			dirLight.color = new Color(1.0f, 1.0f, 0.8f);
			dirLight.intensity = 0.8f;

			Vector3 vibe = Random.insideUnitSphere;
			water.transform.position = new Vector3(0f, 190f, 0f) + vibe;
			transform.position = new Vector3(0f, (timer - 15f) * 5f, 0f) + vibe;

		}
		else
		{
			if(!endMessageShown)
			{
				Messenger.SetMessage("Well, admittedly, your day started out pretty crappy, with your submarine breaking down and all of those gods yammering at you, but hey, you've raised the lost city of Atlantis! I think that counts as a pretty good day overall, don't you think? And wow, being able to fly! That is going to be pretty awesome! Yeah. Today was a great day! You just had to scratch beneath the surface of what you saw!");
				player.gameIsOver = true;
				endMessageShown = true;
			}
		}
	}
}
