﻿using UnityEngine;
using System.Collections.Generic;
using System.Linq;

public class HUDGUI : MonoBehaviour 
{
	public Player player;
	public Texture barTexture;
	public GUISkin skin;
	public Texture[] keyIcons;
	public Texture pauseIcon;

	private Prison[] prisons;

	public bool isPaused;

	void Start()
	{
	}

	void Update()
	{
		if(Input.GetKeyDown(KeyCode.P))
		{
			isPaused ^= true;
			Time.timeScale = isPaused ? 0.0f : 1.0f;
		}
	}

	void OnGUI()
	{
		if(prisons == null)
		{
			prisons = new Prison[5];
			for(int i = 0; i < 5; i++)
			{
				prisons[i] = FindObjectsOfType<Prison>().Where(j => j.keyType == i + 1).FirstOrDefault();
			}
		}

		float scaleFactor = Screen.height * 0.001f;
		GUI.matrix = Matrix4x4.TRS (new Vector3(Screen.width * 0.5f, Screen.height * 0.5f, 0.0f), Quaternion.identity, new Vector3 (scaleFactor, scaleFactor, 1.0f)); 
		Vector2 bottomLeft = new Vector2(-((float)Screen.width / (float)Screen.height) * 500f, 500f);

		if(skin)
		{
			GUI.skin = skin;
		}
		else
		{
			Debug.Log("GUI skin object missing.");
		}
	
		if(!player.gameIsWon)
		{
			float percent = player.airTime / 720f;
			GUI.color = Color.yellow;
			GUI.DrawTexture(new Rect(-500f, 400f, 1000f * percent, 25f), barTexture);
			GUI.color = Color.grey;
			GUI.DrawTexture(new Rect(-500f + 1000f * percent, 400f, 1000f - (1000f * percent), 25f), barTexture);
			GUI.color = Color.white;
			SCGUIUtility.ShadowLabel(new Rect(300f, 430f, 200f, 50f), FormatTime(player.airTime), "LabelRight");
			SCGUIUtility.ShadowLabel(new Rect(-500f, 430f, 200f, 50f), "Oxygen", "LabelLeft");
		}

		if(player.gameIsOver && !player.gameIsWon)
		{
			SCGUIUtility.ShadowLabel(new Rect(-300f, -100f, 600f, 100f), "Game Over", "Title");
			if(GUI.Button(new Rect(-200f, 100f, 400f, 200f), "Respawn?"))
			{
				player.Respawn();
			}
		}
		if(player.gameIsOver && player.gameIsWon)
		{
			SCGUIUtility.ShadowLabel(new Rect(-300f, -100f, 600f, 100f), "You Win", "Title");
			if(GUI.Button(new Rect(-200f, 100f, 400f, 200f), "Menu"))
			{
				SceneManager.LoadScene("Title");
			}
		}

		if(isPaused)
		{
			SCGUIUtility.ShadowLabel(new Rect(-300f, -100f, 600f, 100f), "Paused", "Title");
		}

		if(!player.gameIsOver && !player.gameIsWon)
		{
			if(GUI.Button(new Rect(bottomLeft.x, bottomLeft.y - 75f, 75f, 75f), pauseIcon) || Input.GetKeyDown(KeyCode.P))
			{
				isPaused ^= true;
				Time.timeScale = isPaused ? 0.0f : 1.0f;
			}
		}

		if(!player.gameIsWon)
		{
			float currentY = -400f;
			for(int i = 0; i < 6; i++)
			{
				if(IsKeyKnown(i))
				{
					GUI.DrawTexture(new Rect(-600f, currentY, 50f, 50f), keyIcons[i]);
					if(i < 5)
					{
						SCGUIUtility.ShadowLabel(new Rect(-550f, currentY, 50f, 50f), prisons[i].locks.ToString(), "LabelCenter");
					}
					else
					{
						SCGUIUtility.ShadowLabel(new Rect(-550f, currentY, 50f, 50f), (23 - player.keysDelivered).ToString(), "LabelCenter");
					}
					currentY += 50f;
				}
			}

			if(player.currentKey > 0)
			{
				SCGUIUtility.ShadowLabel(new Rect(500f, -450f, 50f, 50f), "Holding: ", "LabelRight");
				GUI.DrawTexture(new Rect(550f, -450f, 50f, 50f), keyIcons[player.currentKey - 1]);
			}
		}
	}

	private bool IsKeyKnown(int index)
	{
		if(index < 5)
		{
			return prisons[index].wasWelcomed;
		}
		else
		{
			return player.keysDelivered >= 22;
		}
	}

	private string FormatTime(float time)
	{
		System.TimeSpan ts = new System.TimeSpan(0, 0, 0, (int)time);
		return string.Format("{0:00}:{1:00}", (int)ts.TotalMinutes, ts.Seconds);
	}
}
