﻿using UnityEngine;
using System.Collections;
using System.Linq;

public class MilquetoastEnd : MonoBehaviour
{
	public GameObject waterPrefab;
	public GameObject splashPrefab;

	public Player player;
	public GameObject submarine;
	public CameraController cameraController;
	public Light dirLight;
	public float timer;
	public bool endMessageShown;
	public bool waterPrefabSpawned;
	public bool splash1Spawned;
	public bool splash2Spawned;



	void Start () 
	{
		player = FindObjectOfType<Player>();
		player.AddAir(720f);
		player.rigidbody.useGravity = false;
		player.gameIsWon = true;
		submarine = GameObject.FindGameObjectWithTag("Submarine");
		cameraController = FindObjectOfType<CameraController>();
		dirLight = FindObjectsOfType<Light>().Where(i => i.type == LightType.Directional).FirstOrDefault();
		timer = 0f;

		var shark = FindObjectOfType<Shark>();
		Destroy(shark.gameObject);
	}
	
	void Update () 
	{
		timer += Time.deltaTime;

		if(!waterPrefabSpawned)
		{
			Instantiate(waterPrefab, new Vector3(-940f, 460f, 0f), Quaternion.identity);
			waterPrefabSpawned = true;
		}

		if(timer < 10f)
		{
			//Let player read message
		}
		else if(timer < 15f)
		{
			submarine.particleSystem.enableEmission = true;
			player.transform.position = new Vector3(2.5f, 28f, -3.5f);
			player.transform.localRotation = Quaternion.Euler(0f, 290f, 0f);
			submarine.transform.position += ((timer - 10f) * new Vector3(-1.5f, 0.7f, 0.0f)) * Time.deltaTime;
			submarine.transform.localRotation = Quaternion.Lerp(submarine.transform.localRotation, Quaternion.Euler(0f, 0f, -20f), Time.deltaTime * 0.2f);
		}
		else if(timer < 25f)
		{
			submarine.transform.position += ((timer - 10f) * new Vector3(-1.5f, 0.7f, 0.0f)) * Time.deltaTime;
			player.transform.position = Vector3.Lerp(player.transform.position,  submarine.transform.position + new Vector3(20f, 0f, -3.5f), Time.deltaTime * 3f);
			player.transform.localRotation = Quaternion.Euler(-10f, 290f, 0f);
		}
		else if(timer < 32f)
		{
			submarine.transform.position += ((timer - 10f) * new Vector3(-4.5f, 2.1f, 0.0f)) * Time.deltaTime;
			player.transform.position = Vector3.Lerp(player.transform.position,  submarine.transform.position + new Vector3(20f, 0f, -3.5f), Time.deltaTime * 0.2f);
			player.transform.localRotation = Quaternion.Euler(-10f, 290f, 0f);
			if(!splash1Spawned && submarine.transform.position.y > 455.8f)
			{
				Instantiate(splashPrefab, submarine.transform.position + new Vector3(-5f, 5f, 0f), Quaternion.Euler(270f, 0f, 45f));
				splash1Spawned = true;
			}

		}
		else if(timer < 36.8f)
		{
			submarine.particleSystem.enableEmission = false;
			submarine.transform.position += ((timer - 10f) * new Vector3(-1.5f, 0.7f, 0.0f)) * Time.deltaTime;
			cameraController.enabled = false;
			RenderSettings.ambientLight = new Color(0.4f, 0.4f, 0.25f);
			RenderSettings.fogDensity = 0.01f;
			dirLight.color = new Color(1.0f, 1.0f, 0.8f);
			dirLight.intensity = 0.8f;

			if(!splash1Spawned && submarine.transform.position.y > 455.8f)
			{
				Instantiate(splashPrefab, submarine.transform.position + new Vector3(-5f, 5f, 0f), splashPrefab.transform.localRotation);
				splash1Spawned = true;
			}

			Camera.main.transform.position = new Vector3(-940f, 490f, -30f);
			Camera.main.transform.localRotation = Quaternion.Euler(40f, 0f, 0f);
		}
		else if(timer < 38f)
		{
			submarine.transform.position += (new Vector3(-2.5f, 2.0f, 0f) - ((timer - 37f) * new Vector3(1.5f, 9.8f, 0.0f))) * Time.deltaTime;
			submarine.transform.localRotation = Quaternion.Lerp(submarine.transform.localRotation, Quaternion.Euler(0f, 0f, 0f), Time.deltaTime * 1f);

			if(!splash2Spawned)
			{
				Instantiate(splashPrefab, submarine.transform.position, splashPrefab.transform.localRotation);
				splash2Spawned = true;
			}
		}
		else if(timer < 40f)
		{
			submarine.transform.position += new Vector3(-4f, 0f, 0f) * Time.deltaTime;
			submarine.transform.localRotation = Quaternion.Lerp(submarine.transform.localRotation, Quaternion.Euler(0f, 0f, 0f), Time.deltaTime * 3f);
		}
		else
		{
			if(!endMessageShown)
			{
				Messenger.SetMessage("Well here you are. Safe. Happy. Comfortable. You go back to your boring life, always wondering what was up with that lost city, and regretting not seizing the moment and exploring more. For the rest of your days, you are haunted by the spectre of a missed destiny. The end.");
				player.gameIsOver = true;
				endMessageShown = true;
			}
		}
	}
}
