﻿using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour 
{
	public Camera insightCamera;
	public AudioClip airLowSound;
	public AudioClip sharkEatSound;
	public AudioClip drownSound;
	public AudioClip swimSound;
	public GameObject milquetoastEnding;
	public GameObject epicEnding;
	public GameObject wallDestroyEffect;

	public float maxTurnSpeed = 60.0f;
	public float turnAcceleration = 100.0f;
	public float walkAcceleration = 12.0f;

	public float heading;
	public float pitch;
	public float turnSpeed;
	public int currentKey;
	public float insightTimer;

	public int keysDelivered;
	public int subPartsDelivered;
	public bool isSharkFriend;
	public bool hasPermanentInsight;
	public bool hasAresRam;
	public bool hasZeusBolt;
	public string[] keyMessages;
	public string[] subPartMessages;
	public bool gameIsOver;
	public bool gameIsWon;
	public float airTime = 30f;
	public float airLowWarningTimer;
	public Ray lastRamRay;
	public float swimSoundCooldown;



	void Update () 
	{
		Debug.DrawRay(lastRamRay.origin, lastRamRay.direction, Color.red);

		if(!gameIsOver && !gameIsWon)
		{
			insightTimer -= Time.deltaTime;
			airTime -= Time.deltaTime;
			airLowWarningTimer -= Time.deltaTime;

			if(airTime < 60f)
			{
				if(airLowWarningTimer < 0f)
				{
					airLowWarningTimer = 10f;
					AudioSource.PlayClipAtPoint(airLowSound, transform.position);
				}
			}

			if(airTime < 0f)
			{
				Suffocate();
			}


			if(hasPermanentInsight || insightTimer > 0f)
			{
				if(!insightCamera.enabled)
				{
					insightCamera.enabled = true;
				}
			}
			else
			{
				if(insightCamera.enabled)
				{
					insightCamera.enabled = false;
				}
			}
			

			var magSqr = transform.position.sqrMagnitude;
			if(magSqr > 8100f)
			{
				var mag = Mathf.Sqrt(magSqr);
				var resistance = -transform.position.normalized * (mag - 90f) * 2f;
				RenderSettings.fogDensity = 0.04f + (mag - 90f) * 0.06f;
				rigidbody.AddForce(resistance, ForceMode.Acceleration);
			}
			else
			{
				RenderSettings.fogDensity = 0.04f;
			}
		}
	}

	public void ApplyMovement(Vector3 movement, float strafe)
	{
		if(!gameIsOver && !gameIsWon)
		{
			float forward = movement.z;
			if(forward < 0.0f)
			{
				forward *= 0.5f;
			}
			
			var newDirection = HeadingPitchToVector(heading, pitch) * forward;
			var strafeDirection = HeadingPitchToVector(heading + 90f, pitch) * strafe;
			newDirection = newDirection + strafeDirection;
			newDirection.y += movement.y;
			var deltaVelocity = newDirection * walkAcceleration; //* Time.deltaTime;
			rigidbody.AddForce(deltaVelocity, ForceMode.Acceleration);

			swimSoundCooldown -= Time.deltaTime;
			if(!newDirection.sqrMagnitude.Approximately(0f))
			{
				if(swimSoundCooldown < 0f)
				{
					AudioSource.PlayClipAtPoint(swimSound, transform.position, 0.5f);
					swimSoundCooldown = 1.2f;
				}
			}

			if(movement.x.Approximately(0.0f))
			{
				turnSpeed = 0.0f;
			}
			else
			{
				turnSpeed += movement.x * turnAcceleration * Time.deltaTime;
				if(turnSpeed > maxTurnSpeed)
				{
					turnSpeed = maxTurnSpeed;
				}
				if(turnSpeed < -maxTurnSpeed)
				{
					turnSpeed = -maxTurnSpeed;
				}
			}

			heading += turnSpeed * Time.deltaTime;
			heading %= 360.0f;

			transform.localRotation = Quaternion.Euler(pitch, heading, 0f);
		}
	}

	public void UseRam()
	{
		if(hasAresRam || hasZeusBolt)
		{
			var newDirection = HeadingPitchToVector(heading, pitch);
			RaycastHit hitInfo;
			lastRamRay = new Ray(transform.position + Vector3.up, newDirection);
			if(Physics.Raycast(lastRamRay, out hitInfo, 2.5f))
			{
				if(hitInfo.collider.name.Contains("Wall"))
				{
					var effect = Instantiate(wallDestroyEffect, hitInfo.collider.transform.position + Vector3.up, Quaternion.identity) as GameObject;
					Destroy(effect, 1.5f);
					Destroy(hitInfo.collider.gameObject);
					hasZeusBolt = false;
					if(!hasAresRam)
					{
						Messenger.SetMessage("Your deific strength fades and you return to normal.");
					}
				}
			}
		}
	}

	public void GrantPower(int power)
	{
		switch(power)
		{
		case 1:
			isSharkFriend = true;
			break;
		case 2:
			hasPermanentInsight = true;
			break;
		case 3:
			hasAresRam = true;
			break;
		case 4:
			walkAcceleration *= 2f;
			break;
		}
	}

	public void GiveOracleInsight()
	{
		Messenger.SetMessage("The oracle allows you to briefly look within to see item locations. Hurry, this temporary insight will not last long.");
		insightTimer = 15f;
	}

	public void GiveZeusBolt()
	{
		Messenger.SetMessage("The power of the gods has made you so strong that you feel like you could punch a wall. Press R to use the power of the gods.");
		hasZeusBolt = true;
	}

	public void GetEaten()
	{
		RenderSettings.fogColor = Color.red;
		RenderSettings.fogDensity = 0.2f;
		Camera.main.backgroundColor = Color.red;
		AudioSource.PlayClipAtPoint(sharkEatSound, transform.position);

		Messenger.SetMessage("You were eaten by the shark. It seems your destiny was no greater than that of the chicken on your dinner table last night.");
		gameIsOver = true;
	}

	public void Suffocate()
	{
		RenderSettings.fogColor = Color.black;
		RenderSettings.fogDensity = 0.2f;
		Camera.main.backgroundColor = Color.black;
		AudioSource.PlayClipAtPoint(drownSound, transform.position);

		Messenger.SetMessage("You ran out of air. Just another victim of the Atlantean tragedy.");
		gameIsOver = true;
	}

	public void PickupKey(int newKeyType)
	{
		currentKey = newKeyType;

		if(newKeyType != 5)
		{
			Messenger.SetMessage(keyMessages[keysDelivered]);

			if(keysDelivered == 22)
			{
				Instantiate(epicEnding);
			}
		}
		else
		{
			Messenger.SetMessage(subPartMessages[subPartsDelivered]);
		}
	}

	public void DeliverKey()
	{
		if(currentKey <= 4)
		{
			keysDelivered++;
			AddAir(120f);
		}
		else
		{
			subPartsDelivered++;
			if(subPartsDelivered == 3)
			{
				gameIsWon = true;
				Instantiate(milquetoastEnding);
			}
		}

		currentKey = 0;

		if(keysDelivered == 22)
		{
			var factory = FindObjectOfType<BuildingFactory>();
			factory.SpawnZeusKey();
		}
	}

	public void AddAir(float time)
	{
		airTime += time;
		if(airTime > 720f)
		{
			airTime = 720f;
		}
	}

	public void Respawn()
	{
		transform.position = new Vector3(2.5f, 28f, -3.5f);
		gameIsOver = false;
		RenderSettings.fogColor = new Color32(75, 181, 223, 255);
		RenderSettings.fogDensity = 0.04f;
		Camera.main.backgroundColor = new Color32(75, 181, 223, 255);
		airTime = 300f;
	}

	private Vector3 HeadingPitchToVector(float heading, float pitch)
	{
		var headingRadians = heading * Mathf.PI / 180.0f;
		var pitchRadians = pitch * Mathf.PI / 180.0f;
		return new Vector3(Mathf.Sin(headingRadians), Mathf.Sin(pitchRadians), Mathf.Cos(headingRadians));
	}
}
