using UnityEngine;
using System.Collections;

public enum TransitionType { FadeIn, FadeOut }

public class SceneTransition : MonoBehaviour 
{
	public Texture2D transitionTexture;
	public float speed = 1.0f;
	public TransitionType transitionType;
	public bool triggered;
	public bool runWhenPaused;
	internal bool completed;
	internal float position = 0.0f;
	private float triggerTime = 0.0f;
	
	void OnGUI()
	{
		if(triggered)
		{
			switch(transitionType)
			{
				case TransitionType.FadeIn:
					if(!completed)
					{
						GUI.depth = -2;
						GUI.color = new Color(1.0f, 1.0f, 1.0f, 1.0f - position);
						GUI.DrawTexture(new Rect(0.0f, 0.0f, Screen.width, Screen.height), transitionTexture);
					}
					break;
				
				case TransitionType.FadeOut:
					GUI.depth = -2;
					GUI.color = new Color(1.0f, 1.0f, 1.0f, position);
					GUI.DrawTexture(new Rect(0.0f, 0.0f, Screen.width, Screen.height), transitionTexture);
					break;
			}
		}
	}

	// Update is called once per frame
	void Update () 
	{
		if(triggered && !completed)
		{
			if(triggerTime == 0.0f)
			{
				triggerTime = Time.realtimeSinceStartup;
			}
			if(Time.timeScale > 0.0f)
			{
				position += speed * Time.deltaTime;
			}
			else
			{
				if(runWhenPaused && triggerTime > 0.0f)
				{
					position = speed * (Time.realtimeSinceStartup - triggerTime);
				}
			}
			if(position >= 1.0f)
			{
				position = 1.0f;
				completed = true;
			}
		}
	}
	
	public void Clear()
	{
		position = 0.0f;
		triggered = false;
		completed = false;
		triggerTime = 0.0f;
	}
}
