using UnityEngine;
using System.Collections.Generic;

static internal class FloatUtility
{
	static internal bool Approximately(this float value, float other)
	{
		//NOTE: Unity's Approximately function doesn't do a stellar job (accuracy or speed wise),
		//and they won't fix due to risk of breaking legacy code
		//return Mathf.Approximately(value, other);

		const float newEpsilon = 0.00001f;
		
		return Mathf.Abs(value - other) < newEpsilon;
	}
	
	public static float Fraction(this float f)
	{
		#if UNITY_IPHONE
		return f % 1.0f;
		#else
		if(f > 9e18f || f < -9e18f)
		{
			return 0f;
		}
		return f - ((long)f);
		#endif
	}

	static internal float GetAngleDifference(this float value, float other)
	{
		return ((NormalizeAngle(NormalizeAngle(value) - NormalizeAngle(other)) + 180.0f) % 360.0f) - 180.0f;
	}
	
	static internal float NormalizeAngle(this float value)
	{
		return Mathf.Atan2(Mathf.Sin(value / 180.0f * Mathf.PI),Mathf.Cos(value / 180.0f * Mathf.PI)) * 180.0f / Mathf.PI; // (((value + 360.0f) % 360.0f) - 360.0f) % 360.0f;
	}

	static internal float Pulse(float frequency, float min, float max, float t)
	{
		return (((Mathf.Sin(frequency * t * Mathf.PI * 2f) + 1f) * 0.5f) * (max - min)) + min;
	}
}
