using UnityEngine;
using System.Collections.Generic;

static internal class ObjectUtility
{
	static internal bool ToBoolean(this object value)
	{
		if(value is bool)
		{
			return (bool)value;
		}
		
		string toParse = value.ToString();
		bool parsed;
		if(bool.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return false;
	}

	static internal byte ToByte(this object value)
	{
		if(value is byte)
		{
			return (byte)value;
		}
		
		string toParse = value.ToString();
		byte parsed;
		if(byte.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}

	static internal Color ToColor(this object value)
	{
		if(value is Color)
		{
			return (Color)value;
		}
		
		return Color.magenta;
	}
	
	static internal float ToFloat(this object value)
	{
		if(value is float)
		{
			return (float)value;
		}
		
		string toParse = value.ToString();
		float parsed;
		if(float.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}
	
	static internal int ToInt(this object value)
	{
		if(value is int)
		{
			return (int)value;
		}
		
		string toParse = value.ToString();
		int parsed;
		if(int.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}
	
	static internal long ToLong(this object value)
	{
		if(value is long || value is int || value is uint)
		{
			return (long)value;
		}
		
		string toParse = value.ToString();
		long parsed;
		if(long.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}
	
	static internal short ToShort(this object value)
	{
		if(value is short)
		{
			return (short)value;
		}
		
		string toParse = value.ToString();
		short parsed;
		if(short.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}

	static internal uint ToUInt(this object value)
	{
		if(value is uint)
		{
			return (uint)value;
		}
		
		string toParse = value.ToString();
		uint parsed;
		if(uint.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}
	
	static internal ulong ToULong(this object value)
	{
		if(value is ulong || value is int || value is uint)
		{
			return (ulong)value;
		}
		
		string toParse = value.ToString();
		ulong parsed;
		if(ulong.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}
	
	static internal ushort ToUShort(this object value)
	{
		if(value is ushort)
		{
			return (ushort)value;
		}
		
		string toParse = value.ToString();
		ushort parsed;
		if(ushort.TryParse(toParse, out parsed))
		{
			return parsed;
		}
		
		return 0;
	}

	static internal Vector3 ToVector3(this object value)
	{
		if(value is Vector3)
		{
			return (Vector3) value;
		}
		
		return Vector3.zero;
	}

	static internal Vector2 ToVector2(this object value)
	{
		if(value is Vector2)
		{
			return (Vector2) value;
		}
		
		return Vector2.zero;
	}

	static internal IntVector3 ToIntVector3(this object value)
	{
		if(value is IntVector3)
		{
			return (IntVector3) value;
		}
		
		return new IntVector3();
	}
	
	static internal string ToString2(this Vector3 value)
	{
		return string.Format("({0:0.00000}, {1:0.00000}, {2:0.00000})", value.x, value.y, value.z);
	}
	
	static internal bool Approximately(this Vector3 value, Vector3 other)
	{
		return Mathf.Approximately(value.x, other.x) &&
			Mathf.Approximately(value.y, other.y) &&
			Mathf.Approximately(value.z, other.z);
	}
}
