using UnityEngine;
using System.Collections;

public class VolumeControl : MonoBehaviour 
{
	public float volume = 1.0f;
	public bool muted = false;
	public Texture2D barTexture;
	public GUISkin skin;
	
	private float showVolumeTimer;
	
	void Start () 
	{
		volume = AudioListener.volume;
		DontDestroyOnLoad(this);
	}
	
	void Update () 
	{
		showVolumeTimer -= Time.deltaTime;

		if(Input.GetKeyDown(KeyCode.M))
		{
			muted ^= true;
			SetVolume();
		}
		
		if(Input.GetKeyDown(KeyCode.LeftBracket))
		{
			volume -= 0.1f;
			if(volume < 0.0f)
			{
				volume = 0.0f;
			}
			SetVolume();
		}
	
		if(Input.GetKeyDown(KeyCode.RightBracket))
		{
			volume += 0.1f;
			if(volume > 1.0f)
			{
				volume = 1.0f;
			}
			SetVolume();
		}
	}
	
	void OnGUI()
	{
		float scaleFactor = Screen.height * 0.001f;
		GUI.matrix = Matrix4x4.TRS (new Vector3(Screen.width * 0.5f, Screen.height * 0.5f, 0.0f), Quaternion.identity, new Vector3 (scaleFactor, scaleFactor, 1.0f)); 

		if(skin)
		{
			GUI.skin = skin;
		}
		else
		{
			Debug.Log("GUI skin object missing.");
		}

		GUI.depth = -10;

		if(showVolumeTimer > 0.0f)
		{
			float volumeDisplay = volume * 10.0f;
			GUI.Label(new Rect(-160.0f, 400.0f, 320.0f, 80.0f), "", "textArea");
			if(muted)
			{
				GUI.Label(new Rect(-145.0f, 415.0f, 290.0f, 40.0f), "-Muted-", "LabelCenter");
			}
			else
			{
				GUI.Label(new Rect(-145.0f, 415.0f, 290.0f, 40.0f), "Volume", "LabelLeft");
				GUI.Label(new Rect(-145.0f, 415.0f, 290.0f, 40.0f), volumeDisplay.ToString("#,##0"), "LabelRight");
			}
			if(barTexture != null)
			{
				float width = 290.0f * volume;
				GUI.DrawTexture(new Rect(-145.0f, 460.0f, width, 10.0f), barTexture);
			}
		}
	}

	void SetVolume()
	{
		if(muted)
		{
			AudioListener.volume = 0.0f;
		}
		else
		{
			AudioListener.volume = volume;
		}
		showVolumeTimer = 5.0f;
	}
}
